<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.parkingpro.nl
 * @since      1.0.0
 *
 * @package    ParkingPro_Booking_Widgets
 * @subpackage ParkingPro_Booking_Widgets/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    ParkingPro_Booking_Widgets
 * @subpackage ParkingPro_Booking_Widgets/public
 * @author     ParkingPro <info@parkingpro.nl>
 */
class ParkingPro_Booking_Widgets_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $parkingpro_booking_widgets    The ID of this plugin.
	 */
	private $parkingpro_booking_widgets;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $parkingpro_booking_widgets       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $parkingpro_booking_widgets, $version ) {

		$this->parkingpro_booking_widgets = $parkingpro_booking_widgets;
		$this->version = $version;

		add_action( 'widgets_init', function() {

		    $widget = new ParkingPro_Booking_Widgets_Widget( $this->parkingpro_booking_widgets, $this->version );
		    \register_widget( $widget );
		});

		if (!session_id())
    		session_start();

		// Store referrer in session
		if( isset($_SERVER['HTTP_REFERER']) )
		{
			$referer_host = parse_url($_SERVER['HTTP_REFERER'], PHP_URL_HOST);

			if(substr($referer_host, 0 - strlen($_SERVER['HTTP_HOST'])) !== $_SERVER['HTTP_HOST']) 
				$_SESSION['parkingpro_http_referer'] = $_SERVER['HTTP_REFERER'];
		}

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		$wp_scripts = wp_scripts();
		
		wp_enqueue_style( 'jquery-ui-theme-smoothness', sprintf( '//ajax.googleapis.com/ajax/libs/jqueryui/%s/themes/smoothness/jquery-ui.css', $wp_scripts->registered['jquery-ui-core']->ver) );
		wp_enqueue_style( 'jquery-timepicker', plugin_dir_url( __FILE__ ) . 'css/jquery.timepicker.min.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'lightslider', plugin_dir_url( __FILE__ ) . 'css/lightslider.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->parkingpro_booking_widgets, plugin_dir_url( __FILE__ ) . 'css/parkingpro-booking-widgets-public.css', array('jquery-timepicker'), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( 'jquery-ui-datepicker' );
		wp_enqueue_script( 'jquery-timepicker', plugin_dir_url( __FILE__ ) . 'js/jquery.timepicker.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'lightslider', plugin_dir_url( __FILE__ ) . 'js/lightslider.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->parkingpro_booking_widgets, plugin_dir_url( __FILE__ ) . 'js/parkingpro-booking-widgets-public.js', array( 'jquery', 'jquery-timepicker' ), $this->version, false );
		wp_localize_script( $this->parkingpro_booking_widgets, 'parkingpro_booking_widgets', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

	}

	/**
	 * Function to retrieve location price for given dates, used via AJAX in front-end
	 *
	 * @since    1.0.0
	 */
	public function pp_get_prices() {

		try 
		{
			if( !$_POST['location_id'] || !$_POST['arrivalDate'] || !$_POST['departureDate'] )
				return false;

			$data = $this->pp_get_location_price($_POST['location_id'], $_POST['arrivalDate'], $_POST['departureDate']);

	        echo json_encode($data);
			exit();
		} 
		catch (Exception $e) 
		{
			error_log($e);
			return [];
		}

	}

	/**
	 * Function to retrieve location price for given dates, used via AJAX in front-end
	 *
	 * @since    1.1.0
	 */
	public function pp_get_location_price($location_id, $arrivalDate, $departureDate) {

		try 
		{
			if( !$location_id || !$arrivalDate || !$departureDate )
				return false;

			$myparkingpro_url = get_option('parkingpro_booking_widgets_myparkingpro_url');

			if( !$myparkingpro_url )
				return false; 

			$url = $myparkingpro_url . '/api/widget/price?locationId=' . $location_id . '&arrivalDate=' . $arrivalDate . '&departureDate=' . $departureDate;
			$response = wp_remote_get( esc_url_raw($url) );

			if( !is_array( $response ) )
				throw new Exception("Error while loading prices", 1);

	        $array = json_decode($response['body'], true);

	        if( $array && $array['id'] ) {
	            
	            $data = [
	            	'currency' => $array['currency'],
	            	'isUnavailable' => $array['isUnavailable'],
	            	'locationStatus' => $array['locationStatus'],
	            	'totalWithTax' => $array['totalWithTax'],
	            	'totalDays' => $array['totalDays']
	            ];

	        }
	        else {

	        	$data = [];

	        }

	    	return $data;
		} 
		catch (Exception $e) 
		{
			error_log($e);
			return [];
		}

	}

	/**
	 * Add shortcode for booking widget.
	 *
	 * @since    1.0.0
	 */
	public function pp_booking_shortcode( $atts ) {

		$plugin = new ParkingPro_Booking_Widgets();

		$variable_names = $plugin->get_variable_names();

		foreach ($variable_names as $var) 
		{

			$meta_key = $var['meta_key'];
			$short = $var['short'];

			// Set variable via shortcode/widget attribute or load from default settings
			if (isset($atts[ $short ]) )
			    ${"$meta_key"} = $atts[ $short ];
			else
				${"$meta_key"} = get_option($meta_key, $var['default_value']);

			if( isset($parkingpro_booking_widgets_filter_services) && is_string($parkingpro_booking_widgets_filter_services) ) 
				$parkingpro_booking_widgets_filter_services = explode(',', $parkingpro_booking_widgets_filter_services);

		}

		// Retrieve list of services
	    if( $parkingpro_booking_widgets_myparkingpro_url && $parkingpro_booking_widgets_display_services === 'display' )
	    	$parkingpro_booking_widgets_myparkingpro_services = $this->get_myparkingpro_services($parkingpro_booking_widgets_myparkingpro_url);
	    else
	    	$parkingpro_booking_widgets_myparkingpro_services = [];

	    // Retrieve list of airports
	    if( $parkingpro_booking_widgets_myparkingpro_url && $parkingpro_booking_widgets_display_airport_dropdown === 'display' )
	    	$parkingpro_booking_widgets_airports = $this->get_myparkingpro_airports($parkingpro_booking_widgets_myparkingpro_url);
	    else
	    	$parkingpro_booking_widgets_airports = [];

	    if( isset( $atts['airport']) )
	    	$parkingpro_booking_widgets_fixed_airport = $atts['airport'];
	    else
	    	$parkingpro_booking_widgets_fixed_airport = NULL;

	    // Determine the dynamic form destination URL
	    if( isset($parkingpro_booking_widgets_widget_destination) && $parkingpro_booking_widgets_widget_destination === 'product_selection' && isset( $parkingpro_booking_widgets_products_page ) && $parkingpro_booking_widgets_products_page !== '' )
		    $form_destination_url = $parkingpro_booking_widgets_products_page;
		elseif( isset( $parkingpro_booking_widgets_booking_page ) && $parkingpro_booking_widgets_booking_page !== '' )
		    $form_destination_url = $parkingpro_booking_widgets_booking_page;
		else
		    $form_destination_url = $parkingpro_booking_widgets_myparkingpro_url . '/reservations/add';

		$parkingpro_booking_widgets_car_options = ['1', '2', '3', '4', '5'];

		$parkingpro_booking_widgets_config = $this->get_myparkingpro_config($parkingpro_booking_widgets_myparkingpro_url);

		return include( plugin_dir_path( __FILE__ ) . 'partials/parkingpro-booking-widgets-public-display.php' );

	}

	/**
	 * Add shortcode for booking iFrame.
	 *
	 * @since    1.0.0
	 */
	public function pp_booking_iframe_shortcode( $atts ) {

		$myparkingpro_url = get_option('parkingpro_booking_widgets_myparkingpro_url');
		$google_analytics_id = get_option('parkingpro_booking_widgets_google_analytics_id');
		$force_single_service = get_option('parkingpro_booking_widgets_force_single_service');
		$remember_widget_fields = get_option('parkingpro_booking_widgets_remember_widget_fields');
		$iframe_destionation = get_option('parkingpro_booking_widgets_iframe_destination');
		$thankyou_page = get_option('parkingpro_booking_widgets_thankyou_page');

		if( !$myparkingpro_url )
			return '';

		$iframe_url = $myparkingpro_url . '/reservations/add?';

		if( isset($atts['hideheader']) && $atts['hideheader'] !== '' )
			$iframe_url .= '&hideHeader';

		if( isset($atts['hidetitle']) && $atts['hidetitle'] !== '' )
			$iframe_url .= '&hideTitle';

		if( isset($_GET['arrivalDate']) && $_GET['arrivalDate'] !== '' )
			$iframe_url .= '&arrivalDate=' . $_GET['arrivalDate'];
		
		if( isset($_GET['arrivalTime']) && $_GET['arrivalTime'] !== '' )
			$iframe_url .= '&arrivalTime=' . $_GET['arrivalTime'];
		else
			$iframe_url .= '&arrivalTime=12:00';

		if( isset($_GET['departureDate']) && $_GET['departureDate'] !== '' )
			$iframe_url .= '&departureDate=' . $_GET['departureDate'];

		if( isset($_GET['departureTime']) && $_GET['departureTime'] !== '' )
			$iframe_url .= '&departureTime=' . $_GET['departureTime'];
		else
			$iframe_url .= '&departureTime=12:00';

		if( isset($_GET['car']) && $_GET['car'] !== '' )
			$iframe_url .= '&startCarCount=' . $_GET['car'];

		if( isset($_GET['locationId']) && $_GET['locationId'] !== '' )
			$iframe_url .= '&locationId=' . $_GET['locationId'];

		if( $force_single_service === 'true' && isset($_GET['locationId']) && $_GET['locationId'] !== '' ) {
			$iframe_url .= '&showLocations=' . $_GET['locationId'];
		}
		elseif( isset($_GET['showLocations']) && $_GET['showLocations'] !== '' ) {
			$iframe_url .= '&showLocations=' . $_GET['showLocations'];
		}
		elseif( isset($atts['showlocations']) && $atts['showlocations'] !== '' ) {
			$iframe_url .= '&showLocations=' . $atts['showlocations'];	
		}

		if( isset($atts['culture']) && $atts['culture'] !== '' ) {
			$iframe_url .= '&culture=' . $atts['culture'];
		}
		elseif( function_exists('icl_object_id') ) {
			if( ICL_LANGUAGE_CODE === 'nl' )
				$iframe_url .= '&culture=nl-NL';
			elseif( ICL_LANGUAGE_CODE === 'de' )
				$iframe_url .= '&culture=de-DE';
			else
				$iframe_url .= '&culture=en-GB';
		}
		else {
			$culture = get_option('parkingpro_booking_widgets_culture');

			if( isset($culture) && $culture !== "" )
				$iframe_url .= '&culture=' . $culture;
		}
		
		if( isset($_SESSION['parkingpro_http_referer']) )
			$iframe_url .= '&referrer=' . urlencode($_SESSION['parkingpro_http_referer']);

		if( $remember_widget_fields === 'true' )
			$this->pp_store_widget_data($_GET);

		if( isset($atts['height']) && $atts['height'] !== '' )
			$iframe_height = $atts['height'];
		else
			$iframe_height = 2200;

		if( isset($atts['width']) && $atts['width'] !== '' )
			$iframe_width = $atts['width'];
		else
			$iframe_width = '100%';

		$html = '<script> 
			window.addEventListener("message", function (e) { 
				if(e.data.event === "parkingPro.domain.reservationAdded") { ';
			
	    if( $iframe_destionation === 'thankyou_page' && $thankyou_page !== null && $thankyou_page !== '' ) {
	    	$html .= ' 
	    		var destination_url = "' . esc_url_raw($thankyou_page) . '";
		    	if( e.data.reservation && e.data.reservation.location ) {
		    		destination_url += "?" + jQuery.param(e.data.reservation);
		    	}
		    	window.location.href = destination_url;
	    	';
	    }
	    else {
	    	$html .= ' 
	    		jQuery("html, body").animate({
	        		scrollTop: jQuery("#parkingpro_booking_widgets_iframe").offset().top
	        	}, 600); 
	        ';
	    }

	    $html .= '}
	        }, false);</script>';
        
        $html .= '<iframe id="parkingpro_booking_widgets_iframe" style="width: ' . esc_attr($iframe_width) . '; padding: 0; border: 0;" src="' . esc_url_raw($iframe_url) . '" width="' . esc_attr($iframe_width) . '" height="' . esc_attr($iframe_height) . '" frameborder="0" scrolling="yes"></iframe>';

        $google_analytics_id = get_option('parkingpro_booking_widgets_google_analytics_id');

        if( $google_analytics_id && strlen($google_analytics_id) > 0 )
	        $html .= $this->pp_booking_iframe_tracking($google_analytics_id);

        return $html;

	}

	/**
	 * Add shortcode for account registration iFrame.
	 *
	 * @since    1.2.5
	 */
	public function pp_booking_iframe_account_registration_shortcode( $atts ) {

		$myparkingpro_url = get_option('parkingpro_booking_widgets_myparkingpro_url');

		if( !$myparkingpro_url )
			return '';

		$iframe_url = $myparkingpro_url . '/account/register?';

		if( isset($atts['hideheader']) && $atts['hideheader'] !== '' )
			$iframe_url .= '&hideHeader';

		if( isset($atts['hidetitle']) && $atts['hidetitle'] !== '' )
			$iframe_url .= '&hideTitle';

		if( isset($atts['culture']) && $atts['culture'] !== '' ) {
			$iframe_url .= '&culture=' . $atts['culture'];
		}
		elseif( function_exists('icl_object_id') ) {
			if( ICL_LANGUAGE_CODE === 'nl' )
				$iframe_url .= '&culture=nl-NL';
			elseif( ICL_LANGUAGE_CODE === 'de' )
				$iframe_url .= '&culture=de-DE';
			else
				$iframe_url .= '&culture=en-GB';
		}
		else {
			$culture = get_option('parkingpro_booking_widgets_culture');

			if( isset($culture) && $culture !== "" )
				$iframe_url .= '&culture=' . $culture;
		}

		if( isset($atts['height']) && $atts['height'] !== '' )
			$iframe_height = $atts['height'];
		else
			$iframe_height = 1500;

		if( isset($atts['width']) && $atts['width'] !== '' )
			$iframe_width = $atts['width'];
		else
			$iframe_width = '100%';

		$html = '<iframe id="parkingpro_booking_widgets_account_registration_iframe" style="width: ' . esc_attr($iframe_width) . '; padding: 0; border: 0;" src="' . esc_url_raw($iframe_url) . '" width="' . esc_attr($iframe_width) . '" height="' . esc_attr($iframe_height) . '" frameborder="0" scrolling="yes"></iframe>';
		
        return $html;

	}

	/**
	 * Add shortcode for account login iFrame.
	 *
	 * @since    1.2.5
	 */
	public function pp_booking_iframe_account_login_shortcode( $atts ) {

		$myparkingpro_url = get_option('parkingpro_booking_widgets_myparkingpro_url');

		if( !$myparkingpro_url )
			return '';

		$iframe_url = $myparkingpro_url . '/account/login?';

		if( isset($atts['hideheader']) && $atts['hideheader'] !== '' )
			$iframe_url .= '&hideHeader';

		if( isset($atts['hidetitle']) && $atts['hidetitle'] !== '' )
			$iframe_url .= '&hideTitle';

		if( isset($atts['culture']) && $atts['culture'] !== '' ) {
			$iframe_url .= '&culture=' . $atts['culture'];
		}
		elseif( function_exists('icl_object_id') ) {
			if( ICL_LANGUAGE_CODE === 'nl' )
				$iframe_url .= '&culture=nl-NL';
			elseif( ICL_LANGUAGE_CODE === 'de' )
				$iframe_url .= '&culture=de-DE';
			else
				$iframe_url .= '&culture=en-GB';
		}
		else {
			$culture = get_option('parkingpro_booking_widgets_culture');

			if( isset($culture) && $culture !== "" )
				$iframe_url .= '&culture=' . $culture;
		}

		if( isset($atts['height']) && $atts['height'] !== '' )
			$iframe_height = $atts['height'];
		else
			$iframe_height = 1500;

		if( isset($atts['width']) && $atts['width'] !== '' )
			$iframe_width = $atts['width'];
		else
			$iframe_width = '100%';

		$html = '<iframe id="parkingpro_booking_widgets_account_login_iframe" style="width: ' . esc_attr($iframe_width) . '; padding: 0; border: 0;" src="' . esc_url_raw($iframe_url) . '" width="' . esc_attr($iframe_width) . '" height="' . esc_attr($iframe_height) . '" frameborder="0" scrolling="yes"></iframe>';
		
        return $html;

	}

	/**
	 * Add shortcode for parking rates iFrame.
	 *
	 * @since    1.2.5
	 */
	public function pp_booking_iframe_parking_rates_shortcode( $atts ) {

		$myparkingpro_url = get_option('parkingpro_booking_widgets_myparkingpro_url');

		if( !$myparkingpro_url )
			return '';

		$iframe_url = $myparkingpro_url . '/parkingrates?';

		if( isset($atts['hideheader']) && $atts['hideheader'] !== '' )
			$iframe_url .= '&hideHeader';

		if( isset($atts['hidetitle']) && $atts['hidetitle'] !== '' )
			$iframe_url .= '&hideTitle';

		if( isset($atts['maxnumberofdays']) && $atts['maxnumberofdays'] !== '' )
			$iframe_url .= '&maxNumberOfDays' . $atts['maxnumberofdays'];

		if( isset($atts['culture']) && $atts['culture'] !== '' ) {
			$iframe_url .= '&culture=' . $atts['culture'];
		}
		elseif( function_exists('icl_object_id') ) {
			if( ICL_LANGUAGE_CODE === 'nl' )
				$iframe_url .= '&culture=nl-NL';
			elseif( ICL_LANGUAGE_CODE === 'de' )
				$iframe_url .= '&culture=de-DE';
			else
				$iframe_url .= '&culture=en-GB';
		}
		else {
			$culture = get_option('parkingpro_booking_widgets_culture');

			if( isset($culture) && $culture !== "" )
				$iframe_url .= '&culture=' . $culture;
		}

		if( isset($atts['height']) && $atts['height'] !== '' )
			$iframe_height = $atts['height'];
		else
			$iframe_height = 2800;

		if( isset($atts['width']) && $atts['width'] !== '' )
			$iframe_width = $atts['width'];
		else
			$iframe_width = '100%';

		$html = '<iframe id="parkingpro_booking_widgets_parking_rates_iframe" style="width: ' . esc_attr($iframe_width) . '; padding: 0; border: 0;" src="' . esc_url_raw($iframe_url) . '" width="' . esc_attr($iframe_width) . '" height="' . esc_attr($iframe_height) . '" frameborder="0" scrolling="yes"></iframe>';
		
        return $html;

	}

	/**
	 * Add shortcode for product selection.
	 *
	 * @since    1.1.0
	 */
	public function pp_product_selection_shortcode( $atts ) {

		$plugin = new ParkingPro_Booking_Widgets();

		$variable_names = $plugin->get_variable_names();

		foreach ($variable_names as $var) 
		{

			$meta_key = $var['meta_key'];
			$short = $var['short'];

			// Set variable via shortcode/widget attribute or load from default settings
			if (isset($atts[ $short ]) )
			    ${"$meta_key"} = $atts[ $short ];
			else
				${"$meta_key"} = get_option($meta_key, $var['default_value']);

			if( $meta_key === 'parkingpro_booking_widgets_filter_services' && is_string($parkingpro_booking_widgets_filter_services) )
				$parkingpro_booking_widgets_filter_services = explode(',', $parkingpro_booking_widgets_filter_services);

		}

		// Retrieve list of services
	    if( $parkingpro_booking_widgets_myparkingpro_url && isset($_GET['airport']) && $_GET['airport'] !== '' )
	    	$parkingpro_booking_widgets_myparkingpro_services = $this->get_myparkingpro_services($parkingpro_booking_widgets_myparkingpro_url, esc_attr($_GET['airport']) );
	    elseif( $parkingpro_booking_widgets_myparkingpro_url )
	    	$parkingpro_booking_widgets_myparkingpro_services = $this->get_myparkingpro_services($parkingpro_booking_widgets_myparkingpro_url);
	    else
	    	$parkingpro_booking_widgets_myparkingpro_services = [];

	    // Get list of tags
	    $parkingpro_booking_widgets_tags = $this->pp_get_services_tags($parkingpro_booking_widgets_myparkingpro_services);

	    // Retrieve list of airports
	    if( $parkingpro_booking_widgets_myparkingpro_url )
	    	$parkingpro_booking_widgets_airports = $this->get_myparkingpro_airports($parkingpro_booking_widgets_myparkingpro_url);
	    else
	    	$parkingpro_booking_widgets_airports = [];

	    // If enabled, store widget data
	    if( $parkingpro_booking_widgets_remember_widget_fields === 'true' )
	    	$this->pp_store_widget_data($_GET);

	    // Determine the form URL
	    if( isset( $parkingpro_booking_widgets_booking_page ) && $parkingpro_booking_widgets_booking_page !== '' )
		    $form_destination_url = $parkingpro_booking_widgets_booking_page;
		else
		    $form_destination_url = $parkingpro_booking_widgets_myparkingpro_url . '/reservations/add';

		// Calculate total number of days
		$arrivalDate = (isset($_GET['arrivalDate']) ? new DateTime(esc_attr($_GET['arrivalDate'])) : null );
		$departureDate = (isset($_GET['departureDate']) ? new DateTime(esc_attr($_GET['departureDate'])) : null );
		$interval = ($arrivalDate !== null && $departureDate !== null ? $arrivalDate->diff($departureDate) : null);
		$totalDays = ($interval !== null ? $interval->days + 1 : null);

		// Get airport name
		$airport_name = NULL;
		$parkingpro_booking_airport_services = [];

		if( isset($_GET['airport']) && $_GET['airport'] !== '' ) {
		    foreach ($parkingpro_booking_widgets_airports as $parkingpro_booking_widgets_airport) 
		    {
		        if( $parkingpro_booking_widgets_airport['id'] === $_GET['airport'] ) {
		            $airport_name = $parkingpro_booking_widgets_airport['name'];
		            break;
		        }
		    }

		    // Create array of services for the selected airport
		    foreach ($parkingpro_booking_widgets_myparkingpro_services as $service) 
		    {
		        if( in_array($service['id'], $parkingpro_booking_widgets_filter_services) ) {
		            if( $service['airport']['id'] === $_GET['airport'] )
		                array_push($parkingpro_booking_airport_services, $service['id']);
		        }
		    }
		}

		return include( plugin_dir_path( __FILE__ ) . 'partials/parkingpro-booking-widgets-public-product-selection-display.php' );

	}

	/**
	 * Retrieve services for a MyParkingPro URL
	 *
	 * @since     1.0.0
	 * @param     string   $myparkingpro_url       The MyParkingPro URL.
	 * @param     string   $airport_guid           GUID of the airport.
	 * @return    array    List of services.
	 */
	public function get_myparkingpro_services($myparkingpro_url, $airport_guid = NULL) {
		
		try 
		{
			$services_list = [];

			$url = $myparkingpro_url . '/api/widget/locations';
			$response = wp_remote_get( esc_url_raw($url) );

			if( !is_array( $response ) )
				throw new Exception("Error while loading locations", 1);

	        $array = json_decode($response['body'], true);

	        if( $array && $array['data'] && count( $array['data'] ) > 0 ) {
	            
	            foreach( $array['data'] as $service ) {

	            	if( $airport_guid !== NULL ) {
	            		if( $airport_guid === $service['airport']['id'] )
	                		array_push($services_list, $service);
	                }
	                else {
	                	array_push($services_list, $service);
	                }
	            }

	        }

	        return $services_list;
		} 
		catch (Exception $e) 
		{
			error_log($e);
			return [];
		}

	}

	/**
	 * Retrieve all airports for a MyParkingPro URL
	 *
	 * @since     1.1.0
	 * @param     string   $myparkingpro_url       The MyParkingPro URL.
	 * @return    array    List of services.
	 */
	public function get_myparkingpro_airports($myparkingpro_url) {
		
		try 
		{
			$airports_list = [];

			$url = $myparkingpro_url . '/api/widget/locations';
			$response = wp_remote_get( esc_url_raw($url) );

			if( !is_array( $response ) )
				throw new Exception("Error while loading locations", 1);

	        $array = json_decode($response['body'], true);

	        if( $array && $array['data'] && count( $array['data'] ) > 0 ) {
	            
	            foreach( $array['data'] as $service ) 
	            {
	            	if( $service['airport']['id'] )
	                	$airports_list[$service['airport']['id']] = $service['airport'];
	            }

	        }

	        return $airports_list;
		} 
		catch (Exception $e) 
		{
			error_log($e);
			return [];
		}

	}

	/**
	 * Generate script HTML for Cross-domain iFrame tracking
	 * See https://www.simoahava.com/analytics/tracking-cross-domain-iframes-upgraded-solution/
	 *
	 * @since    1.2.3
	 * @param    string   $google_analytics_id       Google Analytics ID
	 * @return   string   HTML tracking script
	 */
	public function pp_booking_iframe_tracking($google_analytics_id) {

		$html = '';

		if( !$google_analytics_id )
			return $html;

	    $html .= "<script>
	        var _customTask = function () {
			  var iframeDecorator = {
			    selector: 'iframe#parkingpro_booking_widgets_iframe',
			    attempts: 10,
			    intervalMs: 1000,
			    useAnchor: false
			  };

			  // DO NOT EDIT ANYTHING BELOW THIS LINE
			  var readFromStorage = function (key) {
			    if (!window.Storage) {
			      // From: https://stackoverflow.com/a/15724300/2367037
			      var value = '; ' + document.cookie;
			      var parts = value.split('; ' + key + '=');
			      if (parts.length === 2) {
			        return parts.pop().split(';').shift();
			      }
			    } else {
			      return window.localStorage.getItem(key);
			    }
			  };

			  var writeToStorage = function (key, value, expireDays) {
			    if (!window.Storage) {
			      var expiresDate = new Date();
			      expiresDate.setDate(expiresDate.getDate() + expireDays);
			      document.cookie = key + '=' + value + ';expires=' + expiresDate.toUTCString();
			    } else {
			      window.localStorage.setItem(key, value);
			    }
			  };

			  var globalSendHitTaskName   = '_ga_originalSendHitTask';

			  return function (customTaskModel) {

			    window[globalSendHitTaskName] = window[globalSendHitTaskName] || customTaskModel.get('sendHitTask');

			    // iframeDecorator
			    if (typeof iframeDecorator === 'object' && typeof iframeDecorator.selector === 'string' && typeof iframeDecorator.attempts === 'number' && typeof iframeDecorator.intervalMs === 'number') {
			      var _id_decorateTimer;
			      var _id_count = 0;
			      var _id_ga = window[window['GoogleAnalyticsObject']];
			      var _id_tracker = _id_ga.getByName(customTaskModel.get('name'));
			      var _id_decorateIframe = function () {
			        var _id_iframe = document.querySelector(iframeDecorator.selector);
			        if (_id_iframe !== null && /[?&]_ga=/.test(_id_iframe.src)) {
			          window.clearInterval(_id_decorateTimer);
			          return;
			        }
			        if (_id_iframe === null) {
			          _id_count += 1;
			          if (_id_count === iframeDecorator.attempts) {
			            window.clearInterval(_id_decorateTimer);
			          }
			          return;
			        }
			        window.clearInterval(_id_decorateTimer);
			        _id_iframe.src = (new window.gaplugins.Linker(_id_tracker)).decorate(_id_iframe.src, iframeDecorator.useAnchor);
			      };
			      _id_decorateTimer = window.setInterval(_id_decorateIframe, iframeDecorator.intervalMs);
			    }
			    // /iframeDecorator

			    customTaskModel.set('sendHitTask', function (sendHitTaskModel) {

			      var originalSendHitTaskModel = sendHitTaskModel,
			          originalSendHitTask      = window[globalSendHitTaskName],
			          canSendHit               = true;

			      try {

			        if (canSendHit) {
			          originalSendHitTask(sendHitTaskModel);
			        }

			      } catch(err) {
			        originalSendHitTask(originalSendHitTaskModel);
			      }

			    });

			  };
			};

			(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
		    (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
		    m=s.getElementsByTagName(o)
		    [0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
		    })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

			ga('create', '" . esc_attr($google_analytics_id) . "');
			ga('set', 'customTask', _customTask());
			ga('send', 'pageview');

		</script>";

        return $html;

	}

	/**
	 * Store URL parameters in session
	 *
	 * @since    1.2.6
	 * @param    array    $data       GET parameters from URL
	 */
	public function pp_store_widget_data($data) {

		if( isset($data['arrivalDate']) && $data['arrivalDate'] !== '' )
			$_SESSION['parkingpro_arrivaldate'] = $data['arrivalDate'];

		if( isset($data['arrivalTime']) && $data['arrivalTime'] !== '' )
			$_SESSION['parkingpro_arrivaltime'] = $data['arrivalTime'];

		if( isset($data['departureDate']) && $data['departureDate'] !== '' )
			$_SESSION['parkingpro_departuredate'] = $data['departureDate'];

		if( isset($data['departureTime']) && $data['departureTime'] !== '' )
			$_SESSION['parkingpro_departuretime'] = $data['departureTime'];

		if( isset($data['airport']) && $data['airport'] !== '' )
			$_SESSION['parkingpro_airport'] = $data['airport'];

		if( isset($data['car']) && $data['car'] !== '' )
			$_SESSION['parkingpro_car'] = $data['car'];

		if( isset($data['locationId']) && $data['locationId'] !== '' )
			$_SESSION['parkingpro_locationid'] = $data['locationId'];

	}

	/**
	 * Add shortcode for thank you page.
	 *
	 * @since    1.2.9
	 */
	public function pp_thank_you_shortcode( $atts ) {

		$plugin = new ParkingPro_Booking_Widgets();

		$variable_names = $plugin->get_variable_names();

		foreach ($variable_names as $var) 
		{

			$meta_key = $var['meta_key'];
			$short = $var['short'];

			// Set variable via shortcode/widget attribute or load from default settings
			if (isset($atts[ $short ]) )
			    ${"$meta_key"} = $atts[ $short ];
			else
				${"$meta_key"} = get_option($meta_key, $var['default_value']);
		}

		return include( plugin_dir_path( __FILE__ ) . 'partials/parkingpro-booking-widgets-public-thank-you-display.php' );

	}

	/**
	 * Add shortcode for thank you page.
	 *
	 * @since    1.2.12
	 */
	public function pp_get_services_tags( $services ) {

		$tag_list = [];

		foreach( $services as $service ) 
		{
			if( is_array($service['appliedTags']) && count($service['appliedTags']) > 0 )
			{
				foreach( $service['appliedTags'] as $appliedTag )
				{
					$tag = [
						'tagId' => $appliedTag['tagId'],
						'tagName' => $appliedTag['tagName']
					];

					if( !in_array($tag, $tag_list) )
						array_push($tag_list, $tag);
				}
			}
		}

		usort($tag_list, function ($item1, $item2) {
		    if ($item1['tagName'] == $item2['tagName']) return 0;
		    return $item1['tagName'] < $item2['tagName'] ? -1 : 1;
		});

		return $tag_list;

	}

	/**
	 * Retrieve the config for a MyParkingPro URL
	 *
	 * @since     1.2.15
	 * @param     string   $myparkingpro_url       The MyParkingPro URL.
	 * @return    array    Config object.
	 */
	public function get_myparkingpro_config($myparkingpro_url) {
		
		try 
		{
			$url = $myparkingpro_url . '/api/widget/config';
			$response = wp_remote_get( esc_url_raw($url) );

			if( !is_array( $response ) )
				throw new Exception("Error while loading config", 1);

	        return json_decode($response['body'], true);
		} 
		catch (Exception $e) 
		{
			error_log($e);
			return [];
		}

	}

}
